% new simple reasoning engine that asserts proof steps
% Author(s) : Xinming Ou, Sakthiyuvaraja Sakthivelmurugan, Loai Zomlot
% Copyright (C) 2012, Argus Cybersecurity Lab, Kansas State University

% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

:- import db_query/4, db_prepare/3, db_prepare_execute/3, db_statement_close/1 from db_interface.
:- import concat_atom/2 from string.
:- table reason/1.
:- table timeRange/3.
:-dynamic int/4.
:-dynamic obs/2.
:-dynamic proofStep/4.
:-dynamic arc/3.
:-dynamic nodeMap/2.
:-dynamic nodeNum/1.

table_init:-
	db_query(snips,trunc,['truncate table snips_summ_proofStep'],_P),
	db_query(snips,trunc,['truncate table snips_internal_proofStep'],_P).


createProofStepsFormFile(Query):- 	
	reason(Query),
	insSummProofStepInDB,
	insIntProofStepInDB.
	

%%getting time interval
createProofStepsFormDB(Query,StartTime,EndTime):-
	getSummaryFromDB(StartTime,EndTime),
	table_init,
	!,
	reason(Query),
	insSummProofStepInDB,
	insIntProofStepInDB.

getSummaryFromDB(StartTime,EndTime):-
	dbToXSB(StartTime,EndTime),
	fixXsbDB.

dbToXSB(QstartTime,QendTime):-
	concat_atom(['select * from snips_summary where start_time >= ',QstartTime,' and ', 'end_time <=',QendTime], C),
	db_query(snips,create,[C],[Pred,H1,H2,Mode,SkolemId,SnortRuleID,StartTime,EndTime]),
	Fact ^=.. [Pred,H1,H2],
	assert(int(Fact, Mode, skolem(SkolemId, SnortRuleID), range(StartTime,EndTime))),
	fail.

dbToXSB(_QstartTime,_QendTime).


%%getting all summary table
createProofStepsFormDB(Query):-
	getSummaryFromDB,
	reason(Query),
	table_init,
	insSummProofStepInDB,
	insIntProofStepInDB.

getSummaryFromDB:-
	dbToXSB,
	fixXsbDB.

dbToXSB:-
	db_query(snips,getSumm,['select * from snips_summary'], [Pred,H1,H2,Mode,SkolemId,SnortRuleID,StartTime,EndTime]),
	Fact ^=.. [Pred,H1,H2],
	assert(int(Fact, Mode, skolem(SkolemId, SnortRuleID), range(StartTime,EndTime))),
	fail.

dbToXSB.

fixXsbDB:-
	iterate(int(Fact,Mode,Skolem,TimeRange),fixRow(Fact,Mode,Skolem,TimeRange)).

fixRow(Fact,Mode,Skolem,TimeRange):-
	Fact ^=.. [Pred,H,null],
	!,
	Fact1 ^=.. [Pred,H],
	assert(int(Fact1, Mode, Skolem,TimeRange)),
	retract(int(Fact,Mode,Skolem,TimeRange)).


insSummProofStepInDB:-
	iterate(proofStep(skolem(SkolemId, snort(SnortRuleId), Mode), summarizedFact, int(Fact,Mode, range(StartTime,EndTime)), o),insSummProofStepRowInDB(SkolemId,SnortRuleId,Mode,Fact,StartTime,EndTime)).
	
insSummProofStepRowInDB(SkolemId,SnortRuleId,Mode,Fact,StartTime,EndTime):-
	Fact ^=.. [Pred,H],
	!,
        db_prepare(snips,ins,['insert into snips_summ_proofStep (skolem_id,snort_rule_id,mode,pred,h1,h2,start_time,end_time)  values(?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [SkolemId,SnortRuleId,Mode,Pred,H,'null',StartTime,EndTime], _P),
	db_statement_close(ins).

insSummProofStepRowInDB(SkolemId,SnortRuleId,Mode,Fact,StartTime,EndTime):-
	Fact ^=.. [Pred,H1,H2],
        db_prepare(snips,ins,['insert into snips_summ_proofStep (skolem_id,snort_rule_id,mode,pred,h1,h2,start_time,end_time)  values(?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [SkolemId,SnortRuleId,Mode,Pred,H1,H2,StartTime,EndTime], _P),
	db_statement_close(ins).


insIntProofStepInDB:-
	iterate(proofStep(int(LFact, LMode, range(LStartTime,LEndTime)), RuleId, int(RFact, RMode, range(RStartTime,REndTime)), Direction),insIntProofStepRowInDB(LFact,LMode,LStartTime,LEndTime,RuleId,RFact,RMode,RStartTime,REndTime,Direction)).
	

insIntProofStepRowInDB(LFact,LMode,LStartTime,LEndTime,RuleId,RFact,RMode,RStartTime,REndTime,Direction):-
	LFact ^=.. [LPred,LH],
	RFact ^=.. [RPred,RH],
	!,
        db_prepare(snips,ins,['insert into snips_internal_proofStep (left_pred,left_h1,left_h2,left_mode,left_start_time,left_end_time,rule_Id,right_pred,right_h1,right_h2,right_mode,right_start_time,right_end_time,direction)  values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [LPred,LH,'null',LMode,LStartTime,LEndTime,RuleId,RPred,RH,'null',RMode,RStartTime,REndTime,Direction], _P),
	db_statement_close(ins).

insIntProofStepRowInDB(LFact,LMode,LStartTime,LEndTime,RuleId,RFact,RMode,RStartTime,REndTime,Direction):-
	LFact ^=.. [LPred,LH],
	RFact ^=.. [RPred,RH1,RH2],
	!,
        db_prepare(snips,ins,['insert into snips_internal_proofStep (left_pred,left_h1,left_h2,left_mode,left_start_time,left_end_time,rule_Id,right_pred,right_h1,right_h2,right_mode,right_start_time,right_end_time,direction)  values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [LPred,LH,'null',LMode,LStartTime,LEndTime,RuleId,RPred,RH1,RH2,RMode,RStartTime,REndTime,Direction], _P),
	db_statement_close(ins).

insIntProofStepRowInDB(LFact,LMode,LStartTime,LEndTime,RuleId,RFact,RMode,RStartTime,REndTime,Direction):-
	LFact ^=.. [LPred,LH1,LH2],
	RFact ^=.. [RPred,RH],
	!,
        db_prepare(snips,ins,['insert into snips_internal_proofStep (left_pred,left_h1,left_h2,left_mode,left_start_time,left_end_time,rule_Id,right_pred,right_h1,right_h2,right_mode,right_start_time,right_end_time,direction)  values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [LPred,LH1,LH2,LMode,LStartTime,LEndTime,RuleId,RPred,RH,'null',RMode,RStartTime,REndTime,Direction], _P),
	db_statement_close(ins).


insIntProofStepRowInDB(LFact,LMode,LStartTime,LEndTime,RuleId,RFact,RMode,RStartTime,REndTime,Direction):-
	LFact ^=.. [LPred,LH1,LH2],
	RFact ^=.. [RPred,RH1,RH2],
        db_prepare(snips,ins,['insert into snips_internal_proofStep (left_pred,left_h1,left_h2,left_mode,left_start_time,left_end_time,rule_Id,right_pred,right_h1,right_h2,right_mode,right_start_time,right_end_time,direction)  values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)']),
        db_prepare_execute(ins, [LPred,LH1,LH2,LMode,LStartTime,LEndTime,RuleId,RPred,RH1,RH2,RMode,RStartTime,REndTime,Direction], _P),
	db_statement_close(ins).

assertProofStep(ProofStep):-
	ProofStep,
	!.
	
assertProofStep(ProofStep):-
	assert(ProofStep).

% proofStep(Known, RuleID, Derived, Direction)

reason(int(Fact, Mode, Time)):-
	  obs(Obs, Time),
	  obsMap(RuleId, obs(Obs), int(Fact), Mode),
	  assertProofStep(proofStep(obs(Obs, Time), RuleId, int(Fact, Mode, Time), o)).

reason(int(Fact, Mode, Time)) :-
	int(Fact, Mode, skolem(ID, SnortRuleID), Time),
	assertProofStep(proofStep(skolem(ID, snort(SnortRuleID), Mode), summarizedFact, int(Fact, Mode, Time), o)).

reason(int(RightFact, Mode, RightTime)):-
	reason(int(LeftFact, Mode1, LeftTime)),
	intRule(RuleId, int(LeftFact), int(RightFact), Direction, Mode2),
	modeJoin(Mode1, Mode2, Mode),
	timeRange(LeftTime, Direction, RightTime),
	assertProofStep(proofStep(int(LeftFact, Mode1, LeftTime), RuleId, int(RightFact, Mode, RightTime), Direction)).

timeRange(LeftTime, f, RightTime) :-
	timeRightOpen(LeftTime, RightTime).

timeRange(LeftTime, b, RightTime) :-
	timeLeftOpen(LeftTime, RightTime).


% Sun, 12 May 2030 16:35:56 GMT = 1904834156
timeRightOpen(range(T1, _T2), range(T1, 1904834156)).
timeLeftOpen(range(_T1, T2), range(0, T2)).

timeOverlap(range(T1a, T2a), range(T1b, T2b), range(T1, T2)) :-
	takeLater(T1a, T1b, T1),
	takeEarlier(T2a, T2b, T2),
	T1 @=< T2.

timeSubsumes(range(T1a, T2a), range(T1b, T2b)) :-
	T1a @=< T1b,
	T2a @>= T2b.

takeLater(T1, T2, T1) :-
	T1 @=< T2.

takeLater(T1, T2, T2) :-
	T2 @< T1.

takeEarlier(T1, T2, T1) :-
	T1 @>= T2.

takeEarlier(T1, T2, T2) :-
	T2 @> T1.

